%% Convolution examples using haemodynamic response function

createGIFs =0;
%% HRFb
load basisSet;
upsampleVal = 2;
hrf=xBF.bf(1:upsampleVal:end,1);
timeDeriv=xBF.bf(1:upsampleVal:end,2);
dispDeriv=xBF.bf(1:upsampleVal:end,3);
RT=xBF.dt;
fhrf = fliplr(hrf); % flipped for convolution
t1 = -40; t2 = 60; % start and end time (end would be 576 for MoAE)
t = t1:RT:t2; % used later
t = t(1:upsampleVal:end);
ti0 = find(t==0);
y = nan(size(t)); % nan so as not to plot zeros outside hrf ...
y(ti0:ti0+length(hrf)-1) = hrf;
plot(t, y / max(y), 'g');
xlim([t1 t2]);
ylim([-0.2 1.2]);
pause(1);


%% Data to convolve
pulse = (t == 0) / max(hrf); % so that max response is 1
step = t >= 0;
T = 42; % (twice as fast as the 42 second blocks (84s period) in MoAE)
slow = sin(2*pi*t/T) .* step;
T = 12; % (illustrates difference in frequency response)
fast = sin(2*pi*t/T) .* step;
sq1 = (sign(slow) + step) / 2;
sq2 = (sign(fast) + step) / 2;
manyPulses = ((abs(t-round(t))<0.001).*(mod(round(t),2)==0).*step)*2/RT/upsampleVal;

data = [pulse; step; manyPulses; sq1; sq2];
D = size(data, 1);
names = {'pulse'; 'step'; 'many events'; 'slow_sq'; 'fast_sq'};

%% Create movies
M = cell(D, 1);
for d = 1:D
    x = data(d, :);
    clf
    plot(t, x);
    xlim([t1 t2]);
    ylim([-0.2 1.2]);
    for f = 1:10 % repeat to hold static on screen for a bit
        M{d}(f) = getframe; % - sorry matlab!
    end
    hold on;
    z = zeros(size(t));
    for tau = length(hrf):length(t);
        y = nan(size(t)); % nan so as not to plot zeros outside hrf ...
        y(tau:-1:1+tau-length(hrf)) = fhrf;
        p = plot(t, y / max(y), 'g'); % (plot hrf with height 1)
        y(isnan(y)) = 0; % ... zero so as not to give nan for z
        z(tau) = sum(x.*y);
        plot(t(tau), z(tau), 'r.');
        M{d}(end+1) = getframe; % - sorry matlab!
        delete(p);
    end
    plot(t, z, 'r');
    for f = 1:15 % repeat to hold static on screen for a bit
        M{d}(end+1) = getframe; % - sorry matlab!
    end
end

%% Play back (just to check frame-rate, but appears unhelpful, see below)
% fps = 36;
% for d = 1:D
%     movie(M{d}, 1, fps);
% end

%% Save as AVI (very big files!)
% for d = 1:D
%     movie2avi(M{d}, [names{d} '.avi'], 'FPS', fps);
% end

%% Save as animated gif

if createGIFs
delay = 0.01; % 1 / fps = 0.278 but that seems to give slow results...
[X map] = rgb2ind(frame2im(M{1}(100)), 256); % get colormap in frame with all colours
for d = 1:D
    X = rgb2ind(frame2im(M{d}(1)), map);
    imwrite(X, map, [names{d} '.gif'], 'gif', 'LoopCount', Inf, 'DelayTime', delay);
    for f = 2:numel(M{d})
        X = rgb2ind(frame2im(M{d}(f)), map);
        imwrite(X, map, [names{d} '.gif'], 'gif', ...
            'WriteMode', 'append', 'DelayTime', delay);
    end
end
end
